/* *********************************************************************
 * Class DcResourceDescriptionHandler
 * 
 * Copyright (c) 2010-2012, German National Library/Deutsche Nationalbibliothek
 * Adickesallee 1, D-60322 Frankfurt am Main, Federal Republic of Germany 
 *
 * This program is free software.
 * For your convenience it is dual licensed.
 * You can redistribute it and/or modify it under the terms of
 * one of the following licenses:
 * 
 * 1.)
 * The GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * You should have received a copy of the GNU General Public License
 * along with this program (gpl-3.0.txt); if not please read
 * http://www.gnu.org/licenses/gpl.html
 * 
 * 2.)
 * The European Union Public Licence as published by
 * The European Commission (executive body of the European Union);
 * either version 1.1 of the License, or (at your option) any later version.
 * You should have received a copy of the European Union Public Licence
 * along with this program (eupl_v1.1_en.pdf); if not please read
 * http://www.osor.eu/eupl
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the above licenses for more details.
 * 
 * Jürgen Kett, Kadir Karaca Kocer -- German National Library
 * 
 **********************************************************************/

/* ******************************************************************
 * CHANGELOG:
 * 
 * 2012-04-03 Port to Maven by Timo Heck & Karaca Kocer
 * 2010-06-09 Code formatting, comments and license text by Karaca Kocer
 * 2010-05-05 Refactoring + new package structure by Karaca Kocer
 * Created on 2010-03-17 by Jürgen Kett
 ********************************************************************/

package org.culturegraph.resolver.api;

import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;

import eu.europeanaconnect.erds.Property;

/**
 * @author Jürgen Kett (Original code)
 * @author Kadir Karaca Kocer (Refactoring, license text, comments & JavaDoc)
 */

public class DcResourceDescriptionHandler extends XmlResourceDescriptionHandler {
    private final static QName XSITYPE = new QName("http://www.w3.org/2001/XMLSchema-instance", "type");
    private static final Log LOGGER = LogFactory.getLog(DcResourceDescriptionHandler.class);

    /**
     * 
     */
    public DcResourceDescriptionHandler() {
        setRecordRootElement(new QName("http://www.openarchives.org/OAI/2.0/oai_dc/", "dc"));
    }

   /**
    * {@inheritDoc}
    */
    @Override
    public List<Property<String, String>> createProperties(XmlObject xmlObject) throws XmlException {

        // DcDocument dcDocument = DcDocument.Factory
        // .parse(xmlObject.getDomNode());

        ArrayList<Property<String, String>> props = new ArrayList<Property<String, String>>();
        XmlCursor cursor = xmlObject.newCursor();
        cursor.toFirstChild();
        while (true) {
            String propertyName = cursor.getName().getPrefix() + ":" + cursor.getName().getLocalPart();

            String xsiType = cursor.getAttributeText(XSITYPE);
            if (xsiType != null) {
                propertyName = xsiType;
            }
            propertyName = propertyName.replaceAll(":", ".");
            propertyName = propertyName.toLowerCase();
            String value = null;
            cursor.toFirstContentToken();
            if (cursor.isText()) {
                value = cursor.getTextValue();
            }
            cursor.toParent();
            if (LOGGER.isDebugEnabled()) LOGGER.debug(propertyName + "= " + value);
            // TODO: do not present urls or urns in the metadata
            props.add(new Property<String, String>(propertyName, value));
            if (!cursor.toNextSibling()) {
                break;
            }
        }
        return props;
    }
}