/* *********************************************************************
 * Class SimpleResourceDescriptionProvider
 * 
 * Copyright (c) 2010-2012, German National Library/Deutsche Nationalbibliothek
 * Adickesallee 1, D-60322 Frankfurt am Main, Federal Republic of Germany 
 *
 * This program is free software.
 * For your convenience it is dual licensed.
 * You can redistribute it and/or modify it under the terms of
 * one of the following licenses:
 * 
 * 1.)
 * The GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * You should have received a copy of the GNU General Public License
 * along with this program (gpl-3.0.txt); if not please read
 * http://www.gnu.org/licenses/gpl.html
 * 
 * 2.)
 * The European Union Public Licence as published by
 * The European Commission (executive body of the European Union);
 * either version 1.1 of the License, or (at your option) any later version.
 * You should have received a copy of the European Union Public Licence
 * along with this program (eupl_v1.1_en.pdf); if not please read
 * http://www.osor.eu/eupl
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the above licenses for more details.
 * 
 * Jürgen Kett, Kadir Karaca Kocer -- German National Library
 * 
 **********************************************************************/

/* ******************************************************************
 * CHANGELOG:
 * 
 * 2012-04-03 Port to Maven by Timo Heck & Karaca Kocer
 * 2010-06-09 Code formatting and license text by Karaca Kocer
 * 2010-05-05 Refactoring + new package structure by Karaca Kocer
 * Created on 2010-03-17 by Jürgen Kett
 ********************************************************************/

package org.culturegraph.resolver.api;

import java.io.IOException;
import java.util.List;
import org.apache.http.HttpResponse;
import org.apache.http.StatusLine;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import eu.europeanaconnect.erds.*;
import eu.europeanaconnect.erds.ResolverException.ResolverExceptionCode;

/**
 * 
 * @author Jürgen Kett (Original code)
 * @author Kadir Karaca Kocer (Refactoring, license text)
 */

public class SimpleResourceDescriptionProvider implements ResourceDescriptionProvider {
    private static final Log LOGGER = LogFactory.getLog(SimpleResourceDescriptionProvider.class);

	private HttpClient httpClient;
	private String requestUrlPattern;
	private List<String> supportedNamespaces;
	private String identifierPattern;
	private String id;
	private String label;
	private ResourceDescriptionHandler resourceDescriptionHandler;
	
	/**
	 * @return Resource description handler
	 */
	public ResourceDescriptionHandler getResourceDescriptionHandler() {
        return this.resourceDescriptionHandler;
    }

    /**
     * @param handler
     */
    public void setResourceDescriptionHandler(ResourceDescriptionHandler handler) {
        this.resourceDescriptionHandler = handler;
    }

    /**
     * @return The label.
     */
    public String getLabel() {
        return this.label;
    }

    /**
     * @param label
     */
    public void setLabel(String label) {
        this.label = label;
    }

    /**
     * {@inheritDoc}
     */
	@Override
	public String getId() {
		return this.id;
	}

	/**
	 * @param id
	 */
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<String> getSupportedNamespaces() {
		return this.supportedNamespaces;
	}

	/**
	 * @param supportedNamespaces
	 */
	public void setSupportedNamespaces(List<String> supportedNamespaces) {
		this.supportedNamespaces = supportedNamespaces;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getIdentifierPattern() {
		return this.identifierPattern;
	}

	/**
	 * @param identifierPattern
	 */
	public void setIdentifierPattern(String identifierPattern) {
		this.identifierPattern = identifierPattern;
	}

	/**
	 * @return Pattern of the URL.
	 */
	public String getRequestUrlPattern() {
		return this.requestUrlPattern;
	}

	/**
	 * @param requestUrlPattern
	 */
	public void setRequestUrlPattern(String requestUrlPattern) {
		this.requestUrlPattern = requestUrlPattern;
	}

	/**
	 * @return org.apache.httpclient
	 */
	public HttpClient getHttpClient() {
		return this.httpClient;
	}

	/**
	 * @param httpClient
	 */
	public void setHttpClient(HttpClient httpClient) {
		this.httpClient = httpClient;
	}

    /**
     * 
	 * @see eu.europeanaconnect.erds.DataProvider
	 */
	@Override
	public ResourceDescription getResponse(ResolverRequest resolverRequest) throws ResolverException {
	    LOGGER.debug("SimpleResourceDescription.getResponse -- Start");
		
    	this.httpClient = new DefaultHttpClient();
        HttpResponse httpResponse = null; 
        String url = getRequestUrl(resolverRequest);
        LOGGER.debug("Contacting the URL = " + url);
        HttpGet getMethod = new HttpGet(url);
        this.httpClient.getParams().setBooleanParameter("http.protocol.handle-redirects", true);
        this.httpClient.getParams().setBooleanParameter("http.protocol.reject-relative-redirect", true);
 
        try {
        	httpResponse = this.httpClient.execute(getMethod);
        	LOGGER.debug("Operation GET successfull");
        } catch (ClientProtocolException e) {
            LOGGER.error("A ClientProtocolException occured!\nStacktrace:\n" + e.getMessage());
        	e.printStackTrace();
            throw new ResolverException(this.id, ResolverExceptionCode.HTTP_PROTOCOL_ERROR, e);
        } catch (IOException e) {
            LOGGER.error("An IOException occured!\nStacktrace:\n" + e.getMessage());
        	e.printStackTrace();
            throw new ResolverException(this.id, ResolverExceptionCode.IO_ERROR, e);
        } catch (RuntimeException e) {
            LOGGER.error("A RuntimeException occured!\nStacktrace:\n" + e.getMessage());
        	e.printStackTrace();
            throw new ResolverException(this.id, ResolverExceptionCode.SEVERE_RUNTIME_ERROR, e);
        } catch (Exception e) {
            LOGGER.error("An Exception occured!\nStacktrace:\n" + e.getMessage());
        	e.printStackTrace();
            throw new ResolverException(this.id, ResolverExceptionCode.UNKNOWN_ERROR, e);
        }
        
        if (LOGGER.isDebugEnabled()) {
            StatusLine statusLine = httpResponse.getStatusLine();
            int statusCode = statusLine.getStatusCode();
            LOGGER.debug("HTTP GET status-code: " + statusCode);        	
        }

		ResourceDescription resourceDescription;
		try {
		    resourceDescription = this.resourceDescriptionHandler.parse(httpResponse.getEntity().getContent());
		    //getResponseBodyAsStream()) is deprecated;
		    LOGGER.debug("HTTP response parsed.");
		} catch (Exception e) {
		    LOGGER.error("Exception during parsing the reponse body.", e);
			throw new ResolverException(this.id, ResolverExceptionCode.REMOTE_RESOLVER_ERROR, e);
		}

		LOGGER.debug("SimpleResourceDescription.getResponse -- End");
		return resourceDescription;
	}

	private String getRequestUrl(ResolverRequest resolverRequest) {
		return this.requestUrlPattern.replace("$identifier", resolverRequest.getIdentifier());
	}
}