/**********************************************************************
 * Class AbstractController
 *  
 * Copyright (c) 2011-2012, German National Library/Deutsche Nationalbibliothek
 * Adickesallee 1, D-60322 Frankfurt am Main, Federal Republic of Germany 
 *
 * This program is free software.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * Thomas Haidlas -- German National Library
 * Kadir Karaca Kocer -- German National Library
 **********************************************************************/

/* ********************************************************************
 * CHANGELOG:
 * 
 * 2012-04-03 Port to Maven by Timo Heck & Karaca Kocer
 * 2011-12-12 License information and JavaDocs by Karaca Kocer
 * Created on 2011-12-08 by Thomas Haidlas -- German National Library
 ********************************************************************/

package org.nbnResolving.resolver.controller.impl;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.culturegraph.resolver.api.MetaResolvingService;
import org.nbnResolving.pidef.HeaderType;
import org.nbnResolving.pidef.MessageType;
import org.nbnResolving.pidef.PidefDocument;
import org.nbnResolving.resolver.Constants;
import org.nbnResolving.resolver.PidefDocumentGenerator;
import org.nbnResolving.resolver.controller.Controller;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.servlet.ModelAndView;

import eu.europeanaconnect.erds.ResolverException;
import eu.europeanaconnect.erds.ResolverRequest;
import eu.europeanaconnect.erds.ResolverResponse;

/**
 * 
 * @author Thomas Haidlas
 */
public abstract class AbstractController implements Controller {
	
    private static final Log LOGGER = LogFactory.getLog(AbstractController.class);
	
	@Autowired
	protected MetaResolvingService metaResolver;
	private String viewName;
	
	public String getViewName() {
		return this.viewName;
	}

	protected void setViewName(String newViewName) {
		this.viewName = newViewName;
	}

	public MetaResolvingService getMetaResolver() {
		return this.metaResolver;
	}

	public void setMetaResolver(MetaResolvingService resolver) {
		this.metaResolver = resolver;
	}

	public ModelAndView resolveId(final ResolverRequest resolverRequest, HttpServletRequest req){
		final PidefDocument pidefDocument = getResponseDoc(resolverRequest, req);		
		return getModelAndView(pidefDocument);
	}
	
	/*new*/
	public ModelAndView resolveId(final ResolverRequest resolverRequest, HttpServletRequest req, String vName){
		final PidefDocument pidefDocument = getResponseDoc(resolverRequest, req);		
		return getModelAndView(pidefDocument, vName);
	}
	
	/*new*/
	private ModelAndView getModelAndView(PidefDocument pidefDocument, String vName) {
		ModelAndView ret = new ModelAndView();
		ret.addObject("doc", pidefDocument);
		ret.setViewName(vName);
		return ret;
	}

	/*new*/
	/**
	 * @param identifier
	 * @param req
	 * @param vName
	 * @return A ModelAndView object to render.
	 */
	public ModelAndView resolveId(final String identifier, final HttpServletRequest req, final String vName) {
		ResolverRequest resolverRequest = new ResolverRequest();
		resolverRequest.setAction("full");
		resolverRequest.setIdentifier(identifier);
		//final PidefDocument pidefDocument = getResponseDoc(resolverRequest, req);		
		//return getModelAndView(pidefDocument);
		return resolveId(resolverRequest, req, vName);
	}
	
	@Override
    public ModelAndView resolveId(final String identifier, final HttpServletRequest req){		
		ResolverRequest resolverRequest = new ResolverRequest();
		resolverRequest.setAction("full");
		resolverRequest.setIdentifier(identifier);
		//final PidefDocument pidefDocument = getResponseDoc(resolverRequest, req);		
		//return getModelAndView(pidefDocument);
		return resolveId(resolverRequest, req);
	}
		
	private PidefDocument getResponseDoc(final ResolverRequest resolverRequest, final HttpServletRequest req){
		ResolverResponse resolverResponse = null;
		try {
			resolverResponse = this.metaResolver.getResponse(resolverRequest);
		} catch (ResolverException e) {
			LOGGER.info("getResponseDoc(): no response for request: " + convertUserRequest(req) + " returning an error header.");
			/*XmlOptions options = new XmlOptions();
			options.setCharacterEncoding("UTF-8");
			options.setSaveNamespacesFirst();
			options.setValidateStrict();
			PidefDocument doc = PidefDocument.Factory.newInstance(options);
			HeaderType header = doc.addNewPidef().addNewHeader();
			*/
			PidefDocument doc = PidefDocumentGenerator.generateNewPidefDocument();
			HeaderType header = doc.getPidef().getHeader();
			header.setStatusCode(Constants.ERROR_DURING_RESOLVING);
			header.setSource("Resolver GUI");
			MessageType message = header.insertNewMessage(0);
			message.setLang("en");
			message.setStringValue(e.getMessage());
			header.setRequest(convertUserRequest(req));
			e.printStackTrace();
			return doc;
		}

		org.nbnResolving.resolver.EpicurResolverResponse rr = null;
		if (resolverResponse instanceof org.nbnResolving.resolver.EpicurResolverResponse) {
			LOGGER.debug("getResponseDoc(): org.nbnResolving.resolver.EpicurResolverResponse found. Casting.");
			rr = (org.nbnResolving.resolver.EpicurResolverResponse) resolverResponse;
		} else {
			rr = new org.nbnResolving.resolver.EpicurResolverResponse();
			//rr.getDoc().getPidef().getData().addNewResolvingInformation().addNewPiInfo().setIdentifier(pi);
			if (resolverResponse.getUrl() != null) {
				LOGGER.debug("getResponseDoc(): Answer of a remote institution. Writing the URL in PIDEF");
				rr.getDoc().getPidef().getHeader().setStatusCode(Constants.REMOTE_RESOLVER_INFORMATION);
				rr.getDoc().getPidef().getHeader().setSource(resolverResponse.getDataProviderId());
				rr.getDoc().getPidef().getData().addNewLinks().addLink(resolverResponse.getUrl());
				//rr.getDoc().getPidef().getData().getLinks().getLinkArray(0);
			} else {
				LOGGER.debug("getResponseDoc(): Answer of a remote institution. No URL found!");
				rr.getDoc().getPidef().getHeader().setStatusCode(Constants.REMOTE_RESOLVER_INFORMATION);
				rr.getDoc().getPidef().getHeader().setSource(resolverResponse.getDataProviderId());
				//TODO: error handling! two separete status codes are needed
			}
		}
		rr.getDoc().getPidef().getHeader().setRequest(convertUserRequest(req));
		
		return rr.getDoc();
	}
	
	private ModelAndView getModelAndView(final PidefDocument pidefDocument) {
		ModelAndView ret = new ModelAndView();
		ret.addObject("doc", pidefDocument);
		ret.setViewName(this.viewName);	
		return ret;
	}
	
	private static String convertUserRequest(final HttpServletRequest req) {
		String url   = req.getRequestURL().toString();
		String query = req.getQueryString();	
		if ((query != null) && !query.isEmpty()) {
			url = url + "?" + query;
		}
		return url;
	}
}
