/**********************************************************************
 * Class XmlController
 *  
 * Copyright (c) 2011-2012, German National Library/Deutsche Nationalbibliothek
 * Adickesallee 1, D-60322 Frankfurt am Main, Federal Republic of Germany 
 *
 * This program is free software.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * Thomas Haidlas -- German National Library
 * Kadir Karaca Kocer -- German National Library
 **********************************************************************/

/* ********************************************************************
 * CHANGELOG:
 * 
 * 2012-04-03 Port to Maven by Timo Heck & Karaca Kocer
 * 2011-12-12 License information and JavaDocs by Karaca Kocer
 * Created on 2011-12-08 by Thomas Haidlas -- German National Library
 ********************************************************************/

package org.nbnResolving.resolver.controller.impl;

import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

/**
 * @author kocer
 *
 */
@Controller
@RequestMapping(value="/xml")
public class XmlController extends AbstractController {
	
	/** Constructor */
	public XmlController(){
		setViewName("xmlView");
	}

	/**
	 * @param identifier Identifier to resolve.
	 * @param req HTTP Request
	 * @return A Spring ModelAndView object to be rendered by Template Engine
	 */
	@RequestMapping("/{identifier:.+}")
	public ModelAndView handleRequest(@PathVariable("identifier") String identifier, HttpServletRequest req){
		return resolveId(identifier, req);
	}

	/**
	 * Resolving methode for identifiers that have / character as Prefix, Suffix delimiter
	 * (Handle, DOI, ARK, ...)
	 * 
	 * @param req HTTP Request
	 * @param prefix A prefix representing the namespace owner
	 * @return A Spring ModelAndView object to be rendered by the appropriate View
	 */
	@RequestMapping("/{prefix}/**")
	public ModelAndView resolvePrefixSuffix(HttpServletRequest req, @PathVariable("prefix") String prefix) {
		String path = req.getServletPath();
		String identifier = path.substring(path.indexOf(prefix));
		return resolveId(identifier, req);
	}

}
